--Backup Class 1
-------------------------------
--A simple full backup statement.
--Notice that it doesn't say FULL anywhere.  It's the default.

BACKUP DATABASE AdventureWorks
TO DISK = 'C:\MyBackups\AW[date].bak'
WITH init, FORMAT

-- INIT is important so you create the same file every time.  Depending on how often you backup this could be really important.  
--You may do fulls more than 
-- once a day or this may also be for log backups too.  it just depends on how you decide to name  your files.  If you don't put a date, 
-- this could be really important.  If you don't use this, you'll have all your backups in a single file.  there's very little use for this.

-- FORMAT-- now this one is interesting because it's important, only you just won't see it very often.  What this does is make sure that the
--file itself is formatted every time.  I was brought into a shop one time where the backups were failing every night when they had been working
-- for months and months on their own.  As it turns out they had something corrupting the file.  I don't remember specifics on it, but the backup
-- would fail sometimes in the middle for no reason at all.  And whenever it did, they couldn't backup again.  Every backup after that failed.
-- Also, something was managing to corrupt the file sometimes even when the file backed up successfully.
-- This is an interesting scenario because it shows you how that really works.  If you INIT the file, it doesn't necessarily create a new file,
-- depending on how you have named them, but it just overwrites the contents of the current file.  That means that the file isn't re-formatted
-- when the new backup is kicked off.  So in the situation, the file header either wasn't written correctly, or it was corrupted afterwards, 
-- so the subsequent backups couldn't succeed.  

-- The key to figuring this out was when I deleted the file, the backups went just fine.  So I added a FORMAT flag to the backup cmd
-- and it never failed on subsequent attempts again.  This however, didn't fix the original corruption, they had to get a driver update
-- to fix that, but these are 2 separate issues really.

-- You can also see this if you're switching the number of files you use.

----------------------------------------------------------------
------------------------Format Flag-----------------------------
----------------------------------------------------------------
-- If you take a backup with a single file the file will be formatted for a single file.
BACKUP DATABASE AdventureWorks
TO DISK = 'C:\AW.bak'
WITH init, format

-- However, when you write to the file again, it's not reformatted so the cmd blows up because it's already formatted for a single file.
-- so even though I've told it to overwrite the file, header doesn't get overwritten.
BACKUP DATABASE AdventureWorks
TO 
DISK = 'C:\AW.bak',
DISK = 'C:\AW2.bak'
WITH init-- , format


----------------------------------------------------------------
------------------------Backup Devices--------------------------
----------------------------------------------------------------
/*

This is one of those things that strikes me as just being antiquated and/or only for beginners because it's so limiting it's of little 
actual use.  The problem with devices is 2-fold.
1.  You have to have the device on the server in order to do a backup or restore.  So that's another level of mgmt.  You can't be dynamic
because if new DBs come on your routine can't pick them up automatically because you'll have to have a device for it.  Of course, you can
make it part of your routine to create the device, but seriously, how cumbersome is that?  And when you restore, you have to have to devices
present as well.  So you have to restore the devices or recreate them.
2.  It also hinders moving backups to other systems (remember lifecycle talk before) because you'll have to create the device on that system as 
well.  Of course, you don't have to, but you've already said you want to work with devices, so if that's gonna be your standard, then you need
to keep it.  And if you're going to change it now and then in order to be more dynamic, then why use them to begin with?
3.  you can't have dynamic files because a device only maps to a single file.  So if you want a dynamic file name, you have to create a new
device for that file each time.  and possibly delete previous devices to keep them from taking over your system.
4.  We'll see later how much easier it is w/o devices.  So my advice is to keep them out of your routines, but you can do whatever you want.



I THOUGHT DEVICES WERE SUPPOSED TO MAKE THINGS EASIER!!!???!!??
*/

BACKUP DATABASE AdventureWorks
TO AW -- this is the dump device.
WITH init, FORMAT

-- To create a device simply use the SP provided.
USE master
GO
EXEC sp_addumpdevice 'disk', 'AW2', 
'C:\MyBackups\AW2.bak';
GO

--You can query all the devices if you like.  If you insisting on managing them this can make it easier if you 
-- use a good naming convention.

SELECT * FROM sys.backup_devices


--Now you can drop the device and the file along with it if you like.
EXEC sp_dropdevice 'AW2', DELFILE
-------------------------------------------------------------
-------------------------------------------------------------
-----------------FileNames and extensions--------------------
-------------------------------------------------------------

BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull.bak'
----WITH INIT
GO 3

--The file extension doesn't matter. 
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull.MyBestBackupEver'
GO 3
--No file extension.
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull'
GO 3
--Use extensions to help protect files from unwanted removal.
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull.Archive--Do-Not-Delete'
GO 3
--Use extensions to help protect DBs from being copied to unprotected systems.
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull.Do-Not-Restore-to-DEV-systems'
GO 3
--Just leave yourself a note on the backup.
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\MyBackups\AdventureWorksFull.I-took-this-backup-because-I-know-the-developer-is-an-idiot-and-will-screwup-his-upgrade'
GO 3

----All of these files will grow each time unless you specifically tell them to overwrite (or initialize in sql terms).
--Uncomment INIT clause.

-------------------------------------------------------------
-------------------------------------------------------------
-----------------Default backup location---------------------
-------------------------------------------------------------

--Of course, with the default backup location that you defined suring setup you don't need to even specify a path.

BACKUP DATABASE [AdventureWorks]
TO DISK = 'AWNoPath.bak'


/*
Now that we've covered the basic backup syntax and switches let's talk about a couple ways you can tune your backups
to get max perf out of them.  Here we're just going to cover the easiest of easy basics but we'll discuss them in 
more detail in later classes.
*/

--BUFFERCOUNT
--MAXTRANSFERSIZE

--USING MAXTRANSFERSIZE
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\mYbACKUPS\AW.bak'
WITH INIT, FORMAT,
MAXTRANSFERSIZE = 1048576--2097152--2097216--4194304

--USING BUFFERCOUNT
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\mYbACKUPS\AW.bak'
WITH INIT, FORMAT,
BUFFERCOUNT = 500

--USING BOTH
BACKUP DATABASE [AdventureWorks]
TO DISK = 'c:\mYbACKUPS\AW.bak'
WITH INIT, FORMAT,
BUFFERCOUNT = 500,
MAXTRANSFERSIZE = 4194304

--NOW ADDING STRIPING
BACKUP DATABASE [AdventureWorks]
TO 
DISK = 'c:\mYbACKUPS\AW1.bak',
DISK = 'c:\mYbACKUPS\AW2.bak'
WITH INIT, FORMAT,
BUFFERCOUNT = 500,
MAXTRANSFERSIZE = 4194304

--NOW ADDING COMPRESSION

BACKUP DATABASE [AdventureWorks]
TO 
DISK = 'c:\mYbACKUPS\AW1.bak',
DISK = 'c:\mYbACKUPS\AW2.bak'
WITH INIT, FORMAT,
BUFFERCOUNT = 500,
MAXTRANSFERSIZE = 2097152,
COMPRESSION

--Nobody wants to alter all their scripts for compression though.  It's the kind of thing that should be done automatically.
--Well, it can be.

sp_configure 'backup compression default', 1

--Let's talk about buffercount again for a min.
--What do you know what to set it at?  What are you already using?

BACKUP DATABASE [AdventureWorks]
TO 
DISK = 'c:\mYbACKUPS\AW1.bak',
DISK = 'c:\mYbACKUPS\AW2.bak',
DISK = 'c:\mYbACKUPS\AW3.bak',
DISK = 'c:\mYbACKUPS\AW4.bak',
DISK = 'c:\mYbACKUPS\AW5.bak',
DISK = 'c:\mYbACKUPS\AW6.bak'
WITH INIT, FORMAT,
--BUFFERCOUNT = 500,
MAXTRANSFERSIZE = 2097152,
COMPRESSION

--Finally we can use a NUL device to measure our success.

BACKUP DATABASE AdventureWorks
TO DISK = 'NUL'
